@@ -77,7 +77,7 @@ class Result(object):
         name (str): name of this result, e.g. new_infections
         npts (int): if values is None, precreate it to be of this length
         scale (str): whether or not the value scales by population size; options are "dynamic", "static", or False
-        color (str or array): default color for plotting (hex or RGB notation)
+        color (str/arr): default color for plotting (hex or RGB notation)
 
     **Example**::
 
@@ -111,11 +111,17 @@ class Result(object):
         return output
 
     def __getitem__(self, *args, **kwargs):
+        ''' To allow e.g. result[5] instead of result.values[5] '''
         return self.values.__getitem__(*args, **kwargs)
 
     def __setitem__(self, *args, **kwargs):
+        ''' To allow e.g. result[:] = 1 instead of result.values[:] = 1 '''
         return self.values.__setitem__(*args, **kwargs)
 
+    def __len__(self):
+        ''' To allow len(result) instead of len(result.values) '''
+        return len(self.values)
+
     @property
     def npts(self):
         return len(self.values)
@@ -209,45 +215,20 @@ class BaseSim(ParsObj):
 
             sim.day('2020-04-05') # Returns 35
         '''
-        # Do not process a day if it's not supplied
-        if day is None:
-            return None
-
-        # Convert to list
-        if sc.isstring(day) or sc.isnumber(day) or isinstance(day, (dt.date, dt.datetime)):
-            day = sc.promotetolist(day) # Ensure it's iterable
-        day.extend(args)
-
-        days = []
-        for d in day:
-            if sc.isnumber(d):
-                days.append(int(d)) # Just convert to an integer
-            else:
-                try:
-                    if sc.isstring(d):
-                        d = sc.readdate(d).date()
-                    elif isinstance(d, dt.datetime):
-                        d = d.date()
-                    d_day = (d - cvm.date(self['start_day'])).days
-                    days.append(d_day)
-                except Exception as E:
-                    errormsg = f'Could not interpret "{d}" as a date: {str(E)}'
-                    raise ValueError(errormsg)
-
-        # Return an integer rather than a list if only one provided
-        if len(days)==1:
-            days = days[0]
-
-        return days
+        return cvm.day(day, *args, start_day=self['start_day'])
 
 
     def date(self, ind, *args, dateformat=None, as_date=False):
         '''
         Convert one or more integer days of simulation time to a date/list of dates --
         by default returns a string, or returns a datetime Date object if as_date is True.
+        See also cv.date(), which provides a partly overlapping set of date conversion
+        features.
 
         Args:
             ind (int, list, or array): the day(s) in simulation time
+            args (list): additional day(s)
+            dateformat (str): the format to return the date in
             as_date (bool): whether to return as a datetime date instead of a string
 
         Returns:
@@ -312,6 +293,11 @@ class BaseSim(ParsObj):
             resdict (dict): dictionary representation of the results
 
         '''
+
+        if not self.results_ready:
+            errormsg = 'Please run the sim before exporting the results'
+            raise RuntimeError(errormsg)
+
         resdict = {}
         resdict['t'] = self.results['t'] # Assume that there is a key for time
 
@@ -427,8 +413,8 @@ class BaseSim(ParsObj):
         '''
         resdict = self.export_results(for_json=False)
         result_df = pd.DataFrame.from_dict(resdict)
-        result_df.index = self.tvec
-        result_df.index.name = 'Day'
+        result_df.index = self.datevec
+        result_df.index.name = 'date'
 
         par_df = pd.DataFrame.from_dict(sc.flattendict(self.pars, sep='_'), orient='index', columns=['Value'])
         par_df.index.name = 'Parameter'
@@ -482,7 +468,7 @@ class BaseSim(ParsObj):
 
         Args:
             filename (str or None): the name or path of the file to save to; if None, uses stored
-            kwargs: passed to makefilepath()
+            kwargs: passed to sc.makefilepath()
 
         Returns:
             filename (str): the validated absolute path to the saved file
@@ -510,7 +496,7 @@ class BaseSim(ParsObj):
             obj = self.shrink(skip_attrs=skip_attrs, in_place=False)
         else:
             obj = self
-        sc.saveobj(filename=filename, obj=obj)
+        cvm.save(filename=filename, obj=obj)
 
         return filename
 
@@ -521,8 +507,8 @@ class BaseSim(ParsObj):
         Load from disk from a gzipped pickle.
 
         Args:
-            filename (str): the name or path of the file to save to
-            kwargs: passed to sc.loadobj()
+            filename (str): the name or path of the file to load from
+            kwargs: passed to cv.load()
 
         Returns:
             sim (Sim): the loaded simulation object
@@ -554,9 +540,9 @@ class BasePeople(sc.prettyobj):
 
         # Handle pars and population size
         pars = sc.mergedicts({'pop_size':0, 'n_days':0}, pars)
-        self.pars = pars
-        self.pop_size = pars['pop_size']
-        self.n_days = pars['n_days']
+        self.pars     = pars # Equivalent to self.set_pars(pars)
+        self.pop_size = int(pars['pop_size'])
+        self.n_days   = int(pars['n_days'])
 
         # Other initialization
         self.t = 0 # Keep current simulation time
@@ -693,6 +679,15 @@ class BasePeople(sc.prettyobj):
         return (self[key]==0).sum()
 
 
+    def set_pars(self, pars):
+        '''
+        Very simple method to re-link the parameters stored in the people object
+        to the sim containing it: included simply for the sake of being explicit.
+        '''
+        self.pars = pars
+        return
+
+
     def keys(self):
         ''' Returns keys for all properties of the people object '''
         return self.meta.all_states[:]
@@ -719,16 +714,19 @@ class BasePeople(sc.prettyobj):
 
 
     def layer_keys(self):
-        ''' Get the available contact keys -- set by beta_layer rather than contacts since only the former is required '''
+        ''' Get the available contact keys -- try contacts first, then beta_layer '''
         try:
-            keys = list(self.pars['beta_layer'].keys())
-        except: # If not initialized
-            keys = []
+            keys = list(self.contacts.keys())
+        except: # If not fully initialized
+            try:
+                keys = list(self.pars['beta_layer'].keys())
+            except: # If not even partially initialized
+                keys = []
         return keys
 
 
-    def index(self):
-        ''' The indices of the array '''
+    def indices(self):
+        ''' The indices of each people array '''
         return np.arange(len(self))
 
 
@@ -1080,7 +1078,7 @@ class Layer(FlexDict):
             n_curr = len(self[key]) # Current number of contacts
             n_new = len(new_arr) # New contacts to add
             n_total = n_curr + n_new # New size
-            self[key].resize(n_total, refcheck=False) # Resize to make room, preserving dtype
+            self[key] = np.resize(self[key], n_total) # Resize to make room, preserving dtype
             self[key][n_curr:] = new_arr # Copy contacts into the layer
         return
 
