@@ -2,7 +2,6 @@
 Miscellaneous functions that do not belong anywhere else
 '''
 
-import os
 import numpy as np
 import pandas as pd
 import pylab as pl
@@ -12,17 +11,18 @@ import scipy.stats as sps
 from . import version as cvver
 
 
-__all__ = ['load_data', 'date', 'daydiff', 'load', 'save', 'savefig', 'get_png_metadata', 'git_info', 'check_version', 'check_save_info', 'get_doubling_time', 'poisson_test']
+__all__ = ['load_data', 'date', 'day', 'daydiff', 'date_range', 'load', 'save', 'savefig', 'get_png_metadata', 'git_info', 'check_version', 'check_save_version', 'get_doubling_time', 'poisson_test', 'compute_gof']
 
 
-def load_data(filename, columns=None, calculate=True, verbose=True, **kwargs):
+def load_data(datafile, columns=None, calculate=True, check_date=True, verbose=True, **kwargs):
     '''
-    Load data for comparing to the model output.
+    Load data for comparing to the model output, either from file or from a dataframe.
 
     Args:
-        filename (str): the name of the file to load (either Excel or CSV)
+        datafile (str or df): if a string, the name of the file to load (either Excel or CSV); if a dataframe, use directly
         columns (list): list of column names (otherwise, load all)
-        calculate (bool): whether or not to calculate cumulative values from daily counts
+        calculate (bool): whether to calculate cumulative values from daily counts
+        check_date (bool): whether to check that a 'date' column is present
         kwargs (dict): passed to pd.read_excel()
 
     Returns:
@@ -30,13 +30,19 @@ def load_data(filename, columns=None, calculate=True, verbose=True, **kwargs):
     '''
 
     # Load data
-    if filename.lower().endswith('csv'):
-        raw_data = pd.read_csv(filename, **kwargs)
-    elif filename.lower().endswith('xlsx'):
-        raw_data = pd.read_excel(filename, **kwargs)
+    if isinstance(datafile, str):
+        if datafile.lower().endswith('csv'):
+            raw_data = pd.read_csv(datafile, **kwargs)
+        elif datafile.lower().endswith('xlsx'):
+            raw_data = pd.read_excel(datafile, **kwargs)
+        else:
+            errormsg = f'Currently loading is only supported from .csv and .xlsx files, not {datafile}'
+            raise NotImplementedError(errormsg)
+    elif isinstance(datafile, pd.DataFrame):
+        raw_data = datafile
     else:
-        errormsg = f'Currently loading is only supported from .csv and .xlsx files, not {filename}'
-        raise NotImplementedError(errormsg)
+        errormsg = f'Could not interpret data {type(datafile)}: must be a string or a dataframe'
+        raise TypeError(errormsg)
 
     # Confirm data integrity and simplify
     if columns is not None:
@@ -59,26 +65,32 @@ def load_data(filename, columns=None, calculate=True, verbose=True, **kwargs):
                     if verbose:
                         print(f'  Automatically adding cumulative column {cum_col} from {col}')
 
-    # Ensure required columns are present
-    if 'date' not in data.columns:
-        errormsg = f'Required column "date" not found; columns are {data.columns}'
-        raise ValueError(errormsg)
-    else:
-        data['date'] = pd.to_datetime(data['date']).dt.date
-
-    data.set_index('date', inplace=True, drop=False) # So sim.data['date'] can still be accessed
+    # Ensure required columns are present and reset the index
+    if check_date:
+        if 'date' not in data.columns:
+            errormsg = f'Required column "date" not found; columns are {data.columns}'
+            raise ValueError(errormsg)
+        else:
+            data['date'] = pd.to_datetime(data['date']).dt.date
+        data.set_index('date', inplace=True, drop=False) # Don't drop so sim.data['date'] can still be accessed
 
     return data
 
 
-def date(obj, *args, **kwargs):
+def date(obj, *args, start_date=None, dateformat=None, as_date=True):
     '''
     Convert a string or a datetime object to a date object. To convert to an integer
-    from the start day, use sim.date() instead.
+    from the start day, it is recommended you supply a start date, or use sim.date()
+    instead; otherwise, it will calculate the date counting days from 2020-01-01.
+    This means that the output of cv.date() will not necessarily match the output
+    of sim.date() for an integer input.
 
     Args:
-        obj (str, date, datetime): the object to convert
+        obj (str, date, datetime, list, array): the object to convert
         args (str, date, datetime): additional objects to convert
+        start_date (str, date, datetime): the starting date, if an integer is supplied
+        dateformat (str): the format to return the date in
+        as_date (bool): whether to return as a datetime date instead of a string
 
     Returns:
         dates (date or list): either a single date object, or a list of them
@@ -86,10 +98,19 @@ def date(obj, *args, **kwargs):
     **Examples**::
 
         cv.date('2020-04-05') # Returns datetime.date(2020, 4, 5)
+        cv.date('2020-04-14', start_date='2020-04-04', as_date=False) # Returns 10
+        cv.date([35,36,37], as_date=False) # Returns ['2020-02-05', '2020-02-06', '2020-02-07']
     '''
-    # Convert to list
+
+    # Convert to list and handle other inputs
+    if isinstance(obj, np.ndarray):
+        obj = obj.tolist() # If it's an array, convert to a list
     obj = sc.promotetolist(obj) # Ensure it's iterable
     obj.extend(args)
+    if dateformat is None:
+        dateformat = '%Y-%m-%d'
+    if start_date is None:
+        start_date = '2020-01-01'
 
     dates = []
     for d in obj:
@@ -100,10 +121,18 @@ def date(obj, *args, **kwargs):
                 d = sc.readdate(d).date()
             elif isinstance(d, dt.datetime):
                 d = d.date()
+            elif sc.isnumber(d):
+                if start_date is None:
+                    errormsg = f'To convert the number {d} to a date, you must supply start_date'
+                    raise ValueError(errormsg)
+                d = date(start_date) + dt.timedelta(days=int(d))
             else:
                 errormsg = f'Cannot interpret {type(d)} as a date, must be date, datetime, or string'
                 raise TypeError(errormsg)
-            dates.append(d)
+            if as_date:
+                dates.append(d)
+            else:
+                dates.append(d.strftime(dateformat))
         except Exception as E:
             errormsg = f'Conversion of "{d}" to a date failed: {str(E)}'
             raise ValueError(errormsg)
@@ -115,10 +144,63 @@ def date(obj, *args, **kwargs):
     return dates
 
 
+def day(obj, *args, start_day=None):
+    '''
+    Convert a string, date/datetime object, or int to a day (int), the number of
+    days since the start day. See also date() and daydiff(). Used primarily via
+    sim.day() rather than directly.
+
+    Args:
+        obj (str, date, int, or list): convert any of these objects to a day relative to the start day
+        args (list): additional days
+        start_day (str or date): the start day; if none is supplied, return days since 2020-01-01.
+
+    Returns:
+        days (int or str): the day(s) in simulation time
+
+    **Example**::
+
+        sim.day('2020-04-05') # Returns 35
+    '''
+
+    # Do not process a day if it's not supplied
+    if obj is None:
+        return None
+    if start_day is None:
+        start_day = '2020-01-01'
+
+    # Convert to list
+    if sc.isstring(obj) or sc.isnumber(obj) or isinstance(obj, (dt.date, dt.datetime)):
+        obj = sc.promotetolist(obj) # Ensure it's iterable
+    obj.extend(args)
+
+    days = []
+    for d in obj:
+        if sc.isnumber(d):
+            days.append(int(d)) # Just convert to an integer
+        else:
+            try:
+                if sc.isstring(d):
+                    d = sc.readdate(d).date()
+                elif isinstance(d, dt.datetime):
+                    d = d.date()
+                d_day = (d - date(start_day)).days # Heavy lifting -- actually compute the day
+                days.append(d_day)
+            except Exception as E:
+                errormsg = f'Could not interpret "{d}" as a date: {str(E)}'
+                raise ValueError(errormsg)
+
+    # Return an integer rather than a list if only one provided
+    if len(days)==1:
+        days = days[0]
+
+    return days
+
+
 def daydiff(*args):
     '''
     Convenience function to find the difference between two or more days. With
-    only one argument, calculate days sin 2020-01-01.
+    only one argument, calculate days since 2020-01-01.
 
     **Example**::
 
@@ -141,6 +223,31 @@ def daydiff(*args):
     return output
 
 
+def date_range(start_date, end_date, inclusive=True, as_date=False, dateformat=None):
+    '''
+    Return a list of dates from the start date to the end date. To convert a list
+    of days (as integers) to dates, use cv.date() instead.
+
+    Args:
+        start_date (int/str/date): the starting date, in any format
+        end_date (int/str/date): the end date, in any format
+        inclusive (bool): if True (default), return to end_date inclusive; otherwise, stop the day before
+        as_date (bool): if True, return a list of datetime.date objects instead of strings
+        dateformat (str): passed to date()
+
+    **Example**::
+
+        dates = cv.date_range('2020-03-01', '2020-04-04')
+    '''
+    start_day = day(start_date)
+    end_day = day(end_date)
+    if inclusive:
+        end_day += 1
+    days = np.arange(start_day, end_day)
+    dates = date(days, as_date=as_date, dateformat=dateformat)
+    return dates
+
+
 def load(*args, **kwargs):
     '''
     Convenience method for sc.loadobj() and equivalent to cv.Sim.load() or
@@ -200,9 +307,13 @@ def get_caller(frame=2, tostring=True):
             else:
                 output = {'filename':fname, 'lineno':lineno}
         except Exception as E:
-            output = f'Calling function information not available ({str(E)})'
+            if tostring:
+                output = f'Calling function information not available ({str(E)})'
+            else:
+                output = {'filename':'N/A', 'lineno':'N/A'}
         return output
 
+
 def savefig(filename=None, dpi=None, comments=None, **kwargs):
     '''
     Wrapper for Matplotlib's savefig() function which automatically stores Covasim
@@ -214,7 +325,6 @@ def savefig(filename=None, dpi=None, comments=None, **kwargs):
         comments (str): additional metadata to save to the figure
         kwargs (dict): passed to savefig()
 
-
     **Example**::
 
         cv.Sim().run(do_plot=True)
@@ -305,7 +415,8 @@ def git_info(filename=None, check=False, comments=None, old_info=None, die=False
 
     # Get git info
     calling_file = sc.makefilepath(get_caller(frame=3, tostring=False)['filename'])
-    cv_info = sc.gitinfo(__file__, verbose=False)
+    cv_info = {'version':cvver.__version__}
+    cv_info.update(sc.gitinfo(__file__, verbose=False))
     caller_info = sc.gitinfo(calling_file, verbose=False)
     caller_info['filename'] = calling_file
     info = {'covasim':cv_info, 'called_by':caller_info}
@@ -355,7 +466,7 @@ def check_version(expected, die=False, verbose=True, **kwargs):
     return compare
 
 
-def check_save_info(expected=None, filename=None, **kwargs):
+def check_save_version(expected=None, filename=None, die=False, verbose=True, **kwargs):
     '''
     A convenience function that bundles check_version with git_info and saves
     automatically to disk from the calling file. The idea is to put this at the
@@ -369,13 +480,13 @@ def check_save_info(expected=None, filename=None, **kwargs):
 
     **Examples**::
 
-        cv.check_save_info()
-        cv.check_save_info('1.3.2', filename='script.gitinfo', comments='This is the main analysis script')
+        cv.check_save_version()
+        cv.check_save_version('1.3.2', filename='script.gitinfo', comments='This is the main analysis script')
     '''
 
     # First, check the version if supplied
     if expected:
-        check_version(expected)
+        check_version(expected, die=die, verbose=verbose)
 
     # Now, check and save the git info
     if filename is None:
@@ -601,3 +712,77 @@ def poisson_test(count1, count2, exposure1=1, exposure2=1, ratio_null=1,
         return zstat_generic2(stat, 1, alternative)
     else:
         return pvalue#, stat
+
+
+def compute_gof(actual, predicted, normalize=True, use_frac=False, use_squared=False, as_scalar='none', eps=1e-9, skestimator=None, **kwargs):
+    '''
+    Calculate the goodness of fit. By default use normalized absolute error, but
+    highly customizable. For example, mean squared error is equivalent to
+    setting normalize=False, use_squared=True, as_scalar='mean'.
+
+    Args:
+        actual      (arr):   array of actual (data) points
+        predicted   (arr):   corresponding array of predicted (model) points
+        normalize   (bool):  whether to divide the values by the largest value in either series
+        use_frac    (bool):  convert to fractional mismatches rather than absolute
+        use_squared (bool):  square the mismatches
+        as_scalar   (str):   return as a scalar instead of a time series: choices are sum, mean, median
+        eps         (float): to avoid divide-by-zero
+        skestimator (str):   if provided, use this scikit-learn estimator instead
+        kwargs      (dict):  passed to the scikit-learn estimator
+
+    Returns:
+        gofs (arr): array of goodness-of-fit values, or a single value if as_scalar is True
+
+    **Examples**::
+
+        x1 = np.cumsum(np.random.random(100))
+        x2 = np.cumsum(np.random.random(100))
+
+        e1 = compute_gof(x1, x2) # Default, normalized absolute error
+        e2 = compute_gof(x1, x2, normalize=False, use_frac=False) # Fractional error
+        e3 = compute_gof(x1, x2, normalize=False, use_squared=True, as_scalar='mean') # Mean squared error
+        e4 = compute_gof(x1, x2, estimator='mean_squared_error') # Scikit-learn's MSE method
+        e5 = compute_gof(x1, x2, as_scalar='median') # Normalized median absolute error -- highly robust
+    '''
+
+    # Custom estimator is supplied: use that
+    if skestimator is not None:
+        try:
+            import sklearn.metrics as sm
+            sklearn_gof = getattr(sm, skestimator) # Shortcut to e.g. sklearn.metrics.max_error
+        except ImportError as E:
+            raise ImportError(f'You must have scikit-learn >=0.22.2 installed: {str(E)}')
+        except AttributeError:
+            raise AttributeError(f'Estimator {skestimator} is not available; see https://scikit-learn.org/stable/modules/model_evaluation.html#scoring-parameter for options')
+        gof = sklearn_gof(actual, predicted, **kwargs)
+        return gof
+
+    # Default case: calculate it manually
+    else:
+        # Key step -- calculate the mismatch!
+        gofs = abs(np.array(actual) - np.array(predicted))
+
+        if normalize and not use_frac:
+            maxmax = max(abs(actual).max(), abs(predicted).max())
+            if maxmax>0:
+                gofs /= maxmax
+
+        if use_frac:
+            if (actual<0).any() or (predicted<0).any():
+                print('Warning: Calculating fractional errors for non-positive quantities is ill-advised!')
+            else:
+                maxvals = np.maximum(actual, predicted) + eps
+                gofs /= maxvals
+
+        if use_squared:
+            gofs = gofs**2
+
+        if as_scalar == 'sum':
+            gofs = np.sum(gofs)
+        elif as_scalar == 'mean':
+            gofs = np.mean(gofs)
+        elif as_scalar == 'median':
+            gofs = np.median(gofs)
+
+        return gofs
\ No newline at end of file
