@@ -7,6 +7,7 @@ import numpy as np
 from . import utils as cvu
 from . import defaults as cvd
 from . import base as cvb
+from . import plotting as cvplt
 
 
 __all__ = ['People']
@@ -434,12 +435,10 @@ class People(cvb.BasePeople):
                 this_trace_time = trace_time[lkey]
 
                 # Find all the contacts of these people
-                nzinds = self.contacts[lkey]['beta'].nonzero()[0] # Find nonzero beta edges
                 inds_list = []
-                for k1,k2 in [['p1','p2'],['p2','p1']]: # Loop over the contact network in both directions
+                for k1,k2 in [['p1','p2'],['p2','p1']]: # Loop over the contact network in both directions -- k1,k2 are the keys
                     in_k1 = np.isin(self.contacts[lkey][k1], inds).nonzero()[0] # Get all the indices of the pairs that each person is in
-                    nz_k1 = np.intersect1d(nzinds, in_k1) # Find the ones that are nonzero
-                    inds_list.append(self.contacts[lkey][k2][nz_k1]) # Find their pairing partner
+                    inds_list.append(self.contacts[lkey][k2][in_k1]) # Find their pairing partner
                 edge_inds = np.unique(np.concatenate(inds_list)) # Find all edges
 
                 # Check contacts
@@ -449,3 +448,23 @@ class People(cvb.BasePeople):
                     self.date_known_contact[contact_inds]  = np.fmin(self.date_known_contact[contact_inds], self.t+this_trace_time)
 
         return
+
+    #%% Analysis methods
+
+    def plot(self, *args, **kwargs):
+        '''
+        Plot statistics of the population -- age distribution, numbers of contacts,
+        and overall weight of contacts (number of contacts multiplied by beta per
+        layer).
+
+        Args:
+            bins (arr): age bins to use (default, 0-100 in one-year bins)
+            width (float): bar width
+            font_size (float): size of font
+            alpha (float): transparency of the plots
+            fig_args (dict): passed to pl.figure()
+            axis_args (dict): passed to pl.subplots_adjust()
+            plot_args (dict): passed to pl.plot()
+        '''
+        fig = cvplt.plot_people(people=self, *args, **kwargs)
+        return fig
