@@ -16,7 +16,7 @@ from . import defaults as cvd
 from . import misc as cvm
 
 
-__all__ = ['plot_sim', 'plot_scens', 'plot_result', 'plot_compare', 'plotly_sim', 'plotly_people', 'plotly_animate']
+__all__ = ['plot_sim', 'plot_scens', 'plot_result', 'plot_compare', 'plot_people', 'plotly_sim', 'plotly_people', 'plotly_animate']
 
 
 #%% Plotting helper functions
@@ -130,8 +130,7 @@ def plot_data(sim, ax, key, scatter_args):
     if sim.data is not None and key in sim.data and len(sim.data[key]):
         this_color = sim.results[key].color
         data_t = (sim.data.index-sim['start_day'])/np.timedelta64(1,'D') # Convert from data date to model output index based on model start date
-        ax.scatter(data_t, sim.data[key], c=[this_color], **scatter_args)
-        ax.scatter(pl.nan, pl.nan, c=[(0,0,0)], label='Data', **scatter_args)
+        ax.scatter(data_t, sim.data[key], c=[this_color], label='Data', **scatter_args)
     return
 
 
@@ -175,9 +174,13 @@ def title_grid_legend(ax, title, grid, commaticks, setylim, legend_args, show_le
     return
 
 
-def reset_ticks(ax, sim, interval, as_dates):
+def reset_ticks(ax, sim, interval, as_dates, dateformat):
     ''' Set the tick marks, using dates by default '''
 
+    # Set the default -- "Mar-01"
+    if dateformat is None:
+        dateformat = '%b-%d'
+
     # Set the x-axis intervals
     if interval:
         xmin,xmax = ax.get_xlim()
@@ -188,7 +191,7 @@ def reset_ticks(ax, sim, interval, as_dates):
 
         @ticker.FuncFormatter
         def date_formatter(x, pos):
-            return (sim['start_day'] + dt.timedelta(days=x)).strftime('%b-%d')
+            return (sim['start_day'] + dt.timedelta(days=x)).strftime(dateformat)
 
         ax.xaxis.set_major_formatter(date_formatter)
         if not interval:
@@ -258,7 +261,7 @@ def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot
             if args.show['data']:
                 plot_data(sim, ax, reskey, args.scatter) # Plot the data
             if args.show['ticks']:
-                reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
+                reset_ticks(ax, sim, interval, as_dates, dateformat) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
         if args.show['interventions']:
             plot_interventions(sim, ax) # Plot the interventions
         if args.show['legend']:
@@ -298,7 +301,7 @@ def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None,
                 if args.show['interventions']:
                     plot_interventions(sim, ax) # Plot the interventions
                 if args.show['ticks']:
-                    reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
+                    reset_ticks(ax, sim, interval, as_dates, dateformat) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
         if args.show['legend']:
             title_grid_legend(ax, title, grid, commaticks, setylim, args.legend, pnum==0) # Configure the title, grid, and legend -- only show legend for first
 
@@ -307,14 +310,16 @@ def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None,
 
 def plot_result(sim, key, fig_args=None, plot_args=None, axis_args=None, scatter_args=None,
                 font_size=18, font_family=None, grid=False, commaticks=True, setylim=True,
-                as_dates=True, dateformat=None, interval=None, color=None, label=None, fig=None):
+                as_dates=True, dateformat=None, interval=None, color=None, label=None, fig=None,
+                do_show=True, do_save=False, fig_path=None):
     ''' Plot a single result -- see Sim.plot_result() for documentation '''
 
     # Handle inputs
+    sep_figs = False # Only one figure
     fig_args  = sc.mergedicts({'figsize':(16,8)}, fig_args)
     axis_args = sc.mergedicts({'top': 0.95}, axis_args)
     args = handle_args(fig_args, plot_args, scatter_args, axis_args)
-    fig, figs, ax = create_figs(args, font_size, font_family, sep_figs=False, fig=fig)
+    fig, figs, ax = create_figs(args, font_size, font_family, sep_figs, fig)
 
     # Gather results
     res = sim.results[key]
@@ -340,9 +345,9 @@ def plot_result(sim, key, fig_args=None, plot_args=None, axis_args=None, scatter
     plot_data(sim, ax, key, args.scatter) # Plot the data
     plot_interventions(sim, ax) # Plot the interventions
     title_grid_legend(ax, res.name, grid, commaticks, setylim, args.legend) # Configure the title, grid, and legend
-    reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
+    reset_ticks(ax, sim, interval, as_dates, dateformat) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
 
-    return fig
+    return tidy_up(fig, figs, sep_figs, do_save, fig_path, do_show)
 
 
 def plot_compare(df, log_scale=True, fig_args=None, plot_args=None, axis_args=None, scatter_args=None,
@@ -389,6 +394,106 @@ def plot_compare(df, log_scale=True, fig_args=None, plot_args=None, axis_args=No
     return fig
 
 
+#%% Other plotting functions
+def plot_people(people, bins=None, width=1.0, font_size=18, alpha=0.6, fig_args=None, axis_args=None, plot_args=None):
+    ''' Plot statistics of a population -- see People.plot() for documentation '''
+
+    # Handle inputs
+    if bins is None:
+        bins = np.arange(0,101)
+
+    # Set defaults
+    color     = [0.1,0.1,0.1] # Color for the age distribution
+    n_rows    = 4 # Number of rows of plots
+    offset    = 0.5 # For ensuring the full bars show up
+    gridspace = 10 # Spacing of gridlines
+    zorder    = 10 # So plots appear on top of gridlines
+
+    # Handle other arguments
+    fig_args  = sc.mergedicts(dict(figsize=(30,22)), fig_args)
+    axis_args = sc.mergedicts(dict(left=0.05, right=0.95, bottom=0.05, top=0.95, wspace=0.3, hspace=0.3), axis_args)
+    plot_args = sc.mergedicts(dict(lw=3, alpha=0.6, markersize=10, c=color, zorder=10), plot_args)
+    pl.rcParams['font.size'] = font_size
+
+    # Compute statistics
+    min_age = min(bins)
+    max_age = max(bins)
+    edges = np.append(bins, np.inf) # Add an extra bin to end to turn them into edges
+    age_counts = np.histogram(people.age, edges)[0]
+
+    # Create the figure
+    fig = pl.figure(**fig_args)
+    pl.subplots_adjust(**axis_args)
+
+    # Plot age histogram
+    pl.subplot(n_rows,2,1)
+    pl.bar(bins, age_counts, color=color, alpha=alpha, width=width, zorder=zorder)
+    pl.xlim([min_age-offset,max_age+offset])
+    pl.xticks(np.arange(0, max_age+1, gridspace))
+    pl.grid(True)
+    pl.xlabel('Age')
+    pl.ylabel('Number of people')
+    pl.title(f'Age distribution ({len(people):n} people total)')
+
+    # Plot cumulative distribution
+    pl.subplot(n_rows,2,2)
+    age_sorted = sorted(people.age)
+    y = np.linspace(0, 100, len(age_sorted)) # Percentage, not hard-coded!
+    pl.plot(age_sorted, y, '-', **plot_args)
+    pl.xlim([0,max_age])
+    pl.ylim([0,100]) # Percentage
+    pl.xticks(np.arange(0, max_age+1, gridspace))
+    pl.yticks(np.arange(0, 101, gridspace)) # Percentage
+    pl.grid(True)
+    pl.xlabel('Age')
+    pl.ylabel('Cumulative proportion (%)')
+    pl.title(f'Cumulative age distribution (mean age: {people.age.mean():0.2f} years)')
+
+    # Calculate contacts
+    lkeys = people.layer_keys()
+    n_layers = len(lkeys)
+    contact_counts = sc.objdict()
+    for lk in lkeys:
+        layer = people.contacts[lk]
+        p1ages = people.age[layer['p1']]
+        p2ages = people.age[layer['p2']]
+        contact_counts[lk] = np.histogram(p1ages, edges)[0] + np.histogram(p2ages, edges)[0]
+
+    # Plot contacts
+    layer_colors = sc.gridcolors(n_layers)
+    share_ax = None
+    for w,w_type in enumerate(['total', 'percapita', 'weighted']): # Plot contacts in different ways
+        for i,lk in enumerate(lkeys):
+            if w_type == 'total':
+                weight = 1
+                total_contacts = 2*len(people.contacts[lk]) # x2 since each contact is undirected
+                ylabel = 'Number of contacts'
+                title = f'Total contacts for layer "{lk}": {total_contacts:n}'
+            elif w_type == 'percapita':
+                weight = np.divide(1.0, age_counts, where=age_counts>0)
+                mean_contacts = 2*len(people.contacts[lk])/len(people)
+                ylabel = 'Per capita number of contacts'
+                title = f'Mean contacts for layer "{lk}": {mean_contacts:0.2f}'
+            elif w_type == 'weighted':
+                weight = people.pars['beta_layer'][lk]*people.pars['beta']
+                total_weight = np.round(weight*len(people.contacts[lk]))
+                ylabel = 'Weighted number of contacts'
+                title = f'Total weight for layer "{lk}": {total_weight:n}'
+
+            ax = pl.subplot(n_rows, n_layers, n_layers*(w+1)+i+1, sharey=share_ax)
+            pl.bar(bins, contact_counts[lk]*weight, color=layer_colors[i], width=width, zorder=zorder, alpha=alpha)
+            pl.xlim([min_age-offset,max_age+offset])
+            pl.xticks(np.arange(0, max_age+1, gridspace))
+            pl.grid(True)
+            pl.xlabel('Age')
+            pl.ylabel(ylabel)
+            pl.title(title)
+            if w_type == 'weighted':
+                share_ax = ax # Update shared axis
+
+    return fig
+
+
 #%% Plotly functions
 
 def get_individual_states(sim):
