@@ -4,7 +4,6 @@ Defines the Sim class, Covasim's core class.
 
 #%% Imports
 import numpy as np
-import pylab as pl
 import sciris as sc
 from . import version as cvv
 from . import utils as cvu
@@ -17,7 +16,7 @@ from . import plotting as cvplt
 from . import interventions as cvi
 from . import analysis as cva
 
-# Specify all externally visible things this file defines
+# Everything in this file is contained in the Sim class
 __all__ = ['Sim']
 
 
@@ -27,15 +26,15 @@ class Sim(cvb.BaseSim):
     number of time points, and the parameters of the simulation.
 
     Args:
-        pars (dict): parameters to modify from their default values
-        datafile (str): filename of (Excel) data file to load, if any
-        datacols (list): list of column names of the data file to load
-        label (str): the name of the simulation (useful to distinguish in batch runs)
-        simfile (str): the filename for this simulation, if it's saved (default: creation date)
-        popfile (str): the filename to load/save the population for this simulation
-        load_pop (bool): whether to load the population from the named file
-        save_pop (bool): whether to save the population to the named file
-        kwargs (dict): passed to make_pars()
+        pars     (dict):   parameters to modify from their default values
+        datafile (str/df): filename of (Excel, CSV) data file to load, or a pandas dataframe of the data
+        datacols (list):   list of column names of the data to load
+        label    (str):    the name of the simulation (useful to distinguish in batch runs)
+        simfile  (str):    the filename for this simulation, if it's saved (default: creation date)
+        popfile  (str):    the filename to load/save the population for this simulation
+        load_pop (bool):   whether to load the population from the named file
+        save_pop (bool):   whether to save the population to the named file
+        kwargs   (dict):   passed to make_pars()
 
     **Examples**::
 
@@ -65,12 +64,11 @@ class Sim(cvb.BaseSim):
         self.results_ready = False    # Whether or not results are ready
 
         # Now update everything
-        self.set_metadata(simfile, label) # Set the simulation date and filename
+        self.set_metadata(simfile, label)  # Set the simulation date and filename
+        self.update_pars(pars, **kwargs)   # Update the parameters, if provided
         self.load_data(datafile, datacols) # Load the data, if provided
-        self.update_pars(pars, **kwargs)             # Update the parameters, if provided
         if self.load_pop:
             self.load_population(popfile)      # Load the population, if provided
-
         return
 
 
@@ -99,11 +97,13 @@ class Sim(cvb.BaseSim):
         return
 
 
-    def load_data(self, datafile=None, datacols=None, **kwargs):
+    def load_data(self, datafile=None, datacols=None, verbose=None, **kwargs):
         ''' Load the data to calibrate against, if provided '''
+        if verbose is None:
+            verbose = self['verbose']
         self.datafile = datafile # Store this
         if datafile is not None: # If a data file is provided, load it
-            self.data = cvm.load_data(filename=datafile, columns=datacols, **kwargs)
+            self.data = cvm.load_data(datafile=datafile, columns=datacols, verbose=verbose, **kwargs)
 
         return
 
@@ -124,7 +124,8 @@ class Sim(cvb.BaseSim):
         self.init_results() # Create the results stucture
         self.init_people(save_pop=self.save_pop, load_pop=self.load_pop, popfile=self.popfile, **kwargs) # Create all the people (slow)
         self.validate_layer_pars() # Once the population is initialized, validate the layer parameters again
-        self.init_interventions()
+        self.init_interventions() # Initialize the interventions
+        self.init_analyzers() # ...and the interventions
         self.set_seed() # Reset the random seed again so the random number stream is consistent
         self.initialized = True
         return
@@ -244,11 +245,12 @@ class Sim(cvb.BaseSim):
             errormsg = f'Population type "{choice}" not available; choices are: {choicestr}'
             raise ValueError(errormsg)
 
-        # Handle interventions
+        # Handle interventions and analyzers
         self['interventions'] = sc.promotetolist(self['interventions'], keepnone=False)
         for i,interv in enumerate(self['interventions']):
             if isinstance(interv, dict): # It's a dictionary representation of an intervention
                 self['interventions'][i] = cvi.InterventionDict(**interv)
+        self['analyzers'] = sc.promotetolist(self['analyzers'], keepnone=False)
 
         # Optionally handle layer parameters
         if validate_layers:
@@ -312,10 +314,11 @@ class Sim(cvb.BaseSim):
         as part of sim.initialize(). Supports loading either saved population
         dictionaries (popdicts, file ending .pop by convention), or ready-to-go
         People objects (file ending .ppl by convention). Either object an also be
-        supplied directly.
+        supplied directly. Once a population file is loaded, it is removed from
+        the Sim object.
 
         Args:
-            popfile (str or obj): name of the file if a string; otherwise, the popdict or People object to load
+            popfile (str or obj): if a string, name of the file; otherwise, the popdict or People object to load
             kwargs (dict): passed to sc.makefilepath()
         '''
         # Set the file path if not is provided
@@ -328,7 +331,7 @@ class Sim(cvb.BaseSim):
             # Load from disk or use directly
             if isinstance(popfile, str): # It's a string, assume it's a filename
                 filepath = sc.makefilepath(filename=popfile, **kwargs)
-                obj = sc.loadobj(filepath)
+                obj = cvm.load(filepath)
                 if self['verbose']:
                     print(f'Loading population from {filepath}')
             else:
@@ -341,7 +344,7 @@ class Sim(cvb.BaseSim):
                 layer_keys   = self.popdict['layer_keys']
             elif isinstance(obj, cvb.BasePeople):
                 self.people = obj
-                self.people.pars = self.pars # Replace the saved parameters with this simulation's
+                self.people.set_pars(self.pars) # Replace the saved parameters with this simulation's
                 n_actual    = len(self.people)
                 layer_keys  = self.people.layer_keys()
             else:
@@ -354,6 +357,7 @@ class Sim(cvb.BaseSim):
                 errormsg = f'Wrong number of people ({n_expected:n} requested, {n_actual:n} actual) -- please change "pop_size" to match or regenerate the file'
                 raise ValueError(errormsg)
             self.reset_layer_pars(force=False, layer_keys=layer_keys) # Ensure that layer keys match the loaded population
+            self.popfile = None # Once loaded, remove to save memory
 
         return
 
@@ -390,29 +394,42 @@ class Sim(cvb.BaseSim):
 
     def init_interventions(self):
         ''' Initialize the interventions '''
-        # Initialize interventions
         for intervention in self['interventions']:
-            if not intervention.initialized:
-                intervention.initialize(self)
+            if isinstance(intervention, cvi.Intervention):
+                if not intervention.initialized:
+                    intervention.initialize(self)
+        return
+
+
+    def init_analyzers(self):
+        ''' Initialize the analyzers '''
+        for analyzer in self['analyzers']:
+            if isinstance(analyzer, cva.Analyzer):
+                if not analyzer.initialized:
+                    analyzer.initialize(self)
         return
 
 
     def rescale(self):
         ''' Dynamically rescale the population -- used during step() '''
         if self['rescale']:
-            t = self.t
             pop_scale = self['pop_scale']
-            current_scale = self.rescale_vec[t]
+            current_scale = self.rescale_vec[self.t]
             if current_scale < pop_scale: # We have room to rescale
-                n_not_sus = self.people.count_not('susceptible')
-                n_people = len(self.people)
-                if n_not_sus / n_people > self['rescale_threshold']: # Check if we've reached point when we want to rescale
-                    max_ratio = pop_scale/current_scale # We don't want to exceed this
-                    scaling_ratio = min(self['rescale_factor'], max_ratio)
-                    self.rescale_vec[t:] *= scaling_ratio # Update the rescaling factor from here on
-                    n = int(n_people*(1.0-1.0/scaling_ratio)) # For example, rescaling by 2 gives n = 0.5*n_people
-                    new_sus_inds = cvu.choose(max_n=n_people, n=n) # Choose who to make susceptible again
-                    self.people.make_susceptible(new_sus_inds)
+                not_sus_inds = self.people.false('susceptible') # Find everyone not susceptible
+                n_not_sus = len(not_sus_inds) # Number of people who are not susceptible
+                n_people = len(self.people) # Number of people overall
+                current_ratio = n_not_sus/n_people # Current proportion not susceptible
+                threshold = self['rescale_threshold'] # Threshold to trigger rescaling
+                if current_ratio > threshold: # Check if we've reached point when we want to rescale
+                    max_ratio = pop_scale/current_scale # We don't want to exceed the total population size
+                    proposed_ratio = max(current_ratio/threshold, self['rescale_factor']) # The proposed ratio to rescale: the rescale factor, unless we've exceeded it
+                    scaling_ratio = min(proposed_ratio, max_ratio) # We don't want to scale by more than the maximum ratio
+                    self.rescale_vec[self.t:] *= scaling_ratio # Update the rescaling factor from here on
+                    n = int(round(n_not_sus*(1.0-1.0/scaling_ratio))) # For example, rescaling by 2 gives n = 0.5*not_sus_inds
+                    choices = cvu.choose(max_n=n_not_sus, n=n) # Choose who to make susceptible again
+                    new_sus_inds = not_sus_inds[choices] # Convert these back into indices for people
+                    self.people.make_susceptible(new_sus_inds) # Make people susceptible again
         return
 
 
@@ -443,9 +460,13 @@ class Sim(cvb.BaseSim):
 
         # Apply interventions
         for intervention in self['interventions']:
-            intervention.apply(self)
-        if self['interv_func'] is not None: # Apply custom intervention function
-            self['interv_func'](self)
+            if isinstance(intervention, cvi.Intervention):
+                intervention.apply(self) # If it's an intervention, call the apply() method
+            elif callable(intervention):
+                intervention(self) # If it's a function, call it directly
+            else:
+                errormsg = f'Intervention {intervention} is neither callable nor an Intervention object'
+                raise ValueError(errormsg)
 
         people.update_states_post() # Check for state changes after interventions
 
@@ -483,7 +504,6 @@ class Sim(cvb.BaseSim):
                 source_inds, target_inds = cvu.compute_infections(beta, sources, targets, betas, rel_trans, rel_sus) # Calculate transmission!
                 people.infect(inds=target_inds, hosp_max=hosp_max, icu_max=icu_max, source=source_inds, layer=lkey) # Actually infect people
 
-
         # Update counts for this time step: stocks
         for key in cvd.result_stocks.keys():
             self.results[f'n_{key}'][t] = people.count(key)
@@ -492,25 +512,35 @@ class Sim(cvb.BaseSim):
         for key,count in people.flows.items():
             self.results[key][t] += count
 
+        # Apply analyzers -- same syntax as interventions
+        for analyzer in self['analyzers']:
+            if isinstance(analyzer, cva.Analyzer):
+                analyzer.apply(self) # If it's an intervention, call the apply() method
+            elif callable(analyzer):
+                analyzer(self) # If it's a function, call it directly
+            else:
+                errormsg = f'Analyzer {analyzer} is neither callable nor an Analyzer object'
+                raise ValueError(errormsg)
+
         # Tidy up
         self.t += 1
         return
 
 
-    def run(self, do_plot=False, until=None, verbose=None, restore_pars=True, reset_seed=True, **kwargs):
+    def run(self, do_plot=False, until=None, restore_pars=True, reset_seed=True, verbose=None, **kwargs):
         '''
         Run the simulation.
 
         Args:
             do_plot (bool): whether to plot
             until (int): day to run until
-            verbose (int): level of detail to print (otherwise uses self['verbose'])
             restore_pars (bool): whether to make a copy of the parameters before the run and restore it after, so runs are repeatable
             reset_seed (bool): whether to reset the random number stream immediately before run
-            kwargs (dict): passed to self.plot()
+            verbose (float): level of detail to print, e.g. 0 = no output, 0.2 = print every 5th day, 1 = print every day
+            kwargs (dict): passed to sim.plot()
 
         Returns:
-            results: the results object (also modifies in-place)
+            results (dict): the results object (also modifies in-place)
         '''
 
         # Initialize
@@ -533,14 +563,15 @@ class Sim(cvb.BaseSim):
         for t in self.tvec:
 
             # Print progress
-            if verbose >= 1:
+            if verbose:
                 elapsed = sc.toc(output=True)
                 simlabel = f'"{self.label}": ' if self.label else ''
                 string = f'  Running {simlabel}{self.datevec[t]} ({t:2.0f}/{self.pars["n_days"]}) ({elapsed:0.2f} s) '
                 if verbose >= 2:
                     sc.heading(string)
-                elif verbose == 1:
-                    sc.progressbar(t+1, self.npts, label=string, length=20, newline=True)
+                else:
+                    if not (t % int(1.0/verbose)):
+                        sc.progressbar(t+1, self.npts, label=string, length=20, newline=True)
 
             # Do the heavy lifting -- actually run the model!
             self.step()
@@ -560,15 +591,24 @@ class Sim(cvb.BaseSim):
         self.finalize(verbose=verbose) # Finalize the results
         sc.printv(f'Run finished after {elapsed:0.2f} s.\n', 1, verbose)
         if restore_pars:
-            self.pars = orig_pars # Restore the original parameters
+            self.restore_pars(orig_pars)
         if do_plot: # Optionally plot
             self.plot(**kwargs)
 
         return self.results
 
 
+    def restore_pars(self, orig_pars):
+        ''' Restore the original parameter values, except for the analyzers '''
+        analyzers = self['analyzers'] # Make a copy so these don't get wiped
+        for key,val in orig_pars.items():
+            self.pars[key] = val # So pointers, e.g. in sim.people, get updated as well
+        self['analyzers'] = analyzers # Restore the analyzers
+        return
+
+
     def finalize(self, verbose=None):
-        ''' Compute final results, likelihood, etc. '''
+        ''' Compute final results '''
 
         # Scale the results
         for reskey in self.result_keys():
@@ -582,14 +622,15 @@ class Sim(cvb.BaseSim):
             self.results[f'cum_{key}'].values[:] = np.cumsum(self.results[f'new_{key}'].values)
         self.results['cum_infections'].values += self['pop_infected']*self.rescale_vec[0] # Include initially infected people
 
-        # Perform calculations on results
-        self.compute_results(verbose=verbose)
-
-        # Convert results to a odicts/objdict to allow e.g. sim.results.diagnoses
-        self.results = sc.objdict(self.results)
-        self.results_ready = True
+        # Final settings
+        self.t -= 1 # During the run, this keeps track of the next step; restore this be the final day of the sim
+        self.results_ready = True # Set this first so self.summary() knows to print the results
         self.initialized = False # To enable re-running
 
+        # Perform calculations on results
+        self.compute_results(verbose=verbose) # Calculate the rest of the results
+        self.results = sc.objdict(self.results) # Convert results to a odicts/objdict to allow e.g. sim.results.diagnoses
+
         return
 
 
@@ -599,7 +640,6 @@ class Sim(cvb.BaseSim):
         self.compute_yield()
         self.compute_doubling()
         self.compute_r_eff()
-        self.compute_likelihood()
         self.compute_summary(verbose=verbose)
         return
 
@@ -788,71 +828,121 @@ class Sim(cvb.BaseSim):
         return self.results['gen_time']
 
 
-    def compute_likelihood(self, weights=None, verbose=None, eps=1e-16):
-        '''
-        Compute the log-likelihood of the current simulation based on the number
-        of new diagnoses.
-
-        Args:
-            weights (dict): the relative wieght to place on each result
-            verbose (bool): detail to print
-            eps (float): to avoid divide-by-zero errors
-
-        Returns:
-            loglike (float): the log-likelihood of the model given the data
-        '''
-
-        if verbose is None:
-            verbose = self['verbose']
-
-        if weights is None:
-            weights = {}
-
-        if self.data is None:
-            return np.nan
-
-        loglike = 0
-
-        model_dates = self.datevec.tolist()
-
-        for key in set(self.result_keys()).intersection(self.data.columns): # For keys present in both the results and in the data
-            weight = weights.get(key, 1) # Use the provided weight if present, otherwise default to 1
-            for d, datum in self.data[key].iteritems():
-                if np.isfinite(datum):
-                    if d in model_dates:
-                        estimate = self.results[key][model_dates.index(d)]
-                        if np.isfinite(datum) and np.isfinite(estimate):
-                            if (datum == 0) and (estimate == 0):
-                                p = 1.0
-                            else:
-                                p = cvm.poisson_test(datum, estimate)
-                            p = max(p, eps)
-                            logp = pl.log(p)
-                            loglike += weight*logp
-                            sc.printv(f'  {d}, data={datum:3.0f}, model={estimate:3.0f}, log(p)={logp:10.4f}, loglike={loglike:10.4f}', 2, verbose)
-
-            self.results['likelihood'] = loglike
-
-        sc.printv(f'Likelihood: {loglike}', 1, verbose)
-        return loglike
-
-
     def compute_summary(self, verbose=None):
         ''' Compute the summary statistics to display at the end of a run '''
 
         if verbose is None:
             verbose = self['verbose']
 
-        summary = sc.objdict()
-        summary_str = 'Summary:\n'
+        self.summary = sc.objdict()
         for key in self.result_keys():
-            summary[key] = self.results[key][-1]
-            if key.startswith('cum_'):
-                summary_str += f'   {summary[key]:5.0f} {self.results[key].name.lower()}\n'
-        sc.printv(summary_str, 1, verbose)
-        self.summary = summary
+            self.summary[key] = self.results[key][-1]
 
-        return summary
+        if verbose:
+            self.summarize()
+
+        return self.summary
+
+
+    def summarize(self, output=False):
+        ''' Print a brief summary of the simulation '''
+        if self.results_ready:
+            summary_str = 'Simulation summary:\n'
+            for key in self.result_keys():
+                if key.startswith('cum_'):
+                    summary_str += f'   {self.summary[key]:5.0f} {self.results[key].name.lower()}\n'
+
+            if not output:
+                print(summary_str)
+            else:
+                return summary_str
+        else:
+            return self.brief(output=output) # If the simulation hasn't been run, default to the brief summary
+
+
+
+    def brief(self, output=False):
+        ''' Return a one-line description of a sim '''
+
+        if self.results_ready:
+            infections = self.summary['cum_infections']
+            deaths = self.summary['cum_deaths']
+            results = f'{infections:n}⚙, {deaths:n}☠'
+        else:
+            results = 'not run'
+
+        if self.label:
+            label = f'"{self.label}"'
+        else:
+            label = '<no label>'
+
+        start = cvm.date(self['start_day'], as_date=False)
+        if self['end_day']:
+            end = cvm.date(self['end_day'], as_date=False)
+        else:
+            end = cvm.date(self['n_days'], start_date=start)
+
+        pop_size = self['pop_size']
+        pop_type = self['pop_type']
+        string   = f'Sim({label}; {start}—{end}; pop: {pop_size:n} {pop_type}; epi: {results})'
+
+        if not output:
+            print(string)
+        else:
+            return string
+
+
+    def compute_fit(self, output=True, *args, **kwargs):
+        '''
+        Compute the fit between the model and the data. See cv.Fit() for more
+        information.
+
+        Args:
+            output (bool): whether or not to return the TransTree; if not, store in sim.results
+            args   (list): passed to cv.Fit()
+            kwargs (dict): passed to cv.Fit()
+
+        **Example**::
+
+            sim = cv.Sim(datafile=data.csv)
+            sim.run()
+            fit = sim.compute_fit()
+            fit.plot()
+        '''
+        fit = cva.Fit(self, *args, **kwargs)
+        if output:
+            return fit
+        else:
+            self.results.fit = fit
+            return
+
+
+    def make_age_histogram(self, output=True, *args, **kwargs):
+        '''
+        Calculate the age histograms of infections, deaths, diagnoses, etc. See
+        cv.age_histogram() for more information. This can be used alternatively
+        to supplying the age histogram as an analyzer to the sim. If used this
+        way, it can only record the final time point since the states of each
+        person are not saved during the sim.
+
+        Args:
+            output (bool): whether or not to return the age histogram; if not, store in sim.results
+            args   (list): passed to cv.age_histogram()
+            kwargs (dict): passed to cv.age_histogram()
+
+        **Example**::
+
+            sim = cv.Sim()
+            sim.run()
+            agehist = sim.make_age_histogram()
+            agehist.plot()
+        '''
+        agehist = cva.age_histogram(sim=self, *args, **kwargs)
+        if output:
+            return agehist
+        else:
+            self.results.agehist = agehist
+            return
 
 
     def make_transtree(self, output=True, *args, **kwargs):
@@ -911,7 +1001,6 @@ class Sim(cvb.BaseSim):
         Returns:
             fig: Figure handle
 
-
         **Example**::
 
             sim = cv.Sim()
